#include <fstream>
#include <vector>
#include <deque>
#include <queue>

struct Node
{
	Node* left;
	Node* right;
	int leftBorder,rightBorder;
	int index,value;
	Node()
		: left(0),right(0),value(0) {}
};

Node* build(int leftBorder,int rightBorder)
{
	Node* result = new Node;
	result->leftBorder = leftBorder;
	result->rightBorder = rightBorder;
	int middle = (result->leftBorder + result->rightBorder) / 2;
	if(leftBorder != rightBorder)
	{
		result->left = build(leftBorder,middle);
		result->right = build(middle + 1,rightBorder);
		if(result->left->value < result->right->value)
			result->index = result->right->index;
		else
			result->index = result->left->index;
	}
	else
		result->index = middle;
	return result;
}

void update(Node* node,int index,int value)
{
	if(node->leftBorder == node->rightBorder)
		node->value = value;
	else
	{
		int middle = (node->leftBorder + node->rightBorder) / 2;
		if(index <= middle)
			update(node->left,index,value);
		else
			update(node->right,index,value);
		node->value = std::max(node->left->value,node->right->value);
		if(node->left->value < node->right->value)
			node->index = node->right->index;
		else
			node->index = node->left->index;
	}
}

std::pair<int,int> get(Node* node,int leftBorder,int rightBorder)
{
	if(node->leftBorder == leftBorder &&
	   node->rightBorder == rightBorder)
		return std::make_pair(node->value,node->index);
	std::pair<int,int> answer;
	int middle = (node->leftBorder + node->rightBorder) / 2;
	if(leftBorder <= middle)
		answer = std::max(answer,get(node->left,leftBorder,std::min(middle,rightBorder)));
	if(rightBorder > middle)
		answer = std::max(answer,get(node->right,std::max(middle + 1,leftBorder),rightBorder));
	return answer;
}

int main()
{
	std::ifstream in("stones.dat");
	std::ofstream out("stones.sol");
	int n,m;
	in >> n >> m;
	std::vector<std::priority_queue<int,std::deque<int> > > city(m);
	Node* tree = build(0,m - 1);
	for(int _ = 0;_ < n;_++)
	{
		int event;
		in >> event;
		if(event == 1)
		{
			int index,weigth;
			in >> index >> weigth;
			city[index - 1].push(weigth);
			update(tree,index- 1,city[index-1].top());
		}
		else
		{
			std::pair<int,int> current = get(tree,0,m - 1);
			city[current.second].pop();
			update(tree,current.second,city[current.second].top());
			out << current.first << std::endl;
		}
	}
	in.close();
	out.close();
}